using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JsonHelper;
using TeeJee.ProcessHelper;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;
using Json;

public class OperateLogRepo : GLib.Object {
	public string path = "";
	public DateTime date;
	public bool valid = true;

	public Gee.ArrayList<OperateLog?> operateLogs;

	public OperateLogRepo(string dir_path) {
		path = path_combine(dir_path, "operateLog");
	}

	public void read_operatelog_file() {
		log_debug("read_operatelog_file()");

		string operateLog_file = path + "/operate.json";
		stdout.printf(operateLog_file, "\n");
		var f = File.new_for_path(operateLog_file);

		if (f.query_exists()) {

			var parser = new Json.Parser();

			try {
				parser.load_from_file(operateLog_file);
			} catch (Error e) {
				log_error(e.message);
				stdout.printf(e.message);
			}

			var node = parser.get_root();
			// stdout.printf("%s\n",Json.to_string(node,false));
			var config = node.get_object();
			// stdout.printf("%s\n",Json.to_string(config,false));
			if ((node == null) || (config == null)) {
				valid = false;
				return;
			}

			if (config.has_member("log")) {
				log_debug("has_member\n");
				var configlog = config.get_array_member("log");
				uint logLen = 0;
				for (uint a = configlog.get_length(); (a >= 1); a--) {

					var log = configlog.get_object_element(a - 1);
					OperateLog operateLog = new OperateLog();
					string val = json_get_string(log, "date", "");
					if (val.length > 0) {
						DateTime date_utc = new DateTime.from_unix_utc(int64.parse(val));
						DateTime now = new DateTime.now_local();

						if (now.compare(date_utc.add_months(3)) > 0) {
							continue;
						}
						operateLog.date = date_utc.to_local();
					}


					operateLog.user = json_get_string(log, "user", "");
					operateLog.action = json_get_string(log, "action", "");
					operateLog.result = json_get_string(log, "result", "");
					operateLogs.add(operateLog);
					logLen++;
					if (logLen >= 100) {
						break;
					}
				}
				;
			}
			log_debug("loading operateLog from %d found".printf(operateLogs.size));
		} else {
			valid = false;
		}

		log_debug("read_operatelog_file(): exit");
	}

	public bool load_operateLog() {
		operateLogs = new Gee.ArrayList<OperateLog> ();
		log_debug("operateLogsRepo: load_operateLog()");
		// operateLogs
		read_operatelog_file();

		return true;
	}

	public void update_operatelog_file(DateTime dt_created, string user, string result, string action) {
		/* Updates tag and comments */

		try {
			string operateLog_file = path + "/operate.json";
			var f = File.new_for_path(operateLog_file);

			var root = new Json.Node(NodeType.OBJECT);
			if (f.query_exists()) {

				var parser = new Json.Parser();
				try {
					parser.load_from_file(operateLog_file);
				} catch (Error e) {
					log_error(e.message);
				}
				root = parser.get_root();
			} else {
				f.make_directory_with_parents();

				var gen = new Generator();
				var object = new Json.Object();
				root.set_object(object);
				gen.set_root(root);

				var args = new Json.Array();
				object.set_array_member("log", args);
			}

			var config = root.get_object();
			var configlog = config.get_array_member("log");
			var newLog = new Json.Object();

			newLog.set_string_member("date", dt_created.to_utc().to_unix().to_string());
			newLog.set_string_member("user", user);
			newLog.set_string_member("result", result);
			newLog.set_string_member("action", action);

			configlog.add_object_element(newLog);

			if (configlog.get_length() > 120) {
				configlog.remove_element(0);
			}

			var json = new Json.Generator();
			json.pretty = true;
			json.indent = 2;
			root.set_object(config);
			json.set_root(root);
			f.delete();
			json.to_file(operateLog_file);
		} catch (Error e) {
			log_error(e.message);
		}
	}
}