/*
 * LogListBox.vala
 *
 * Copyright 2012-2018 Tony George <teejeetech@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */

using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JsonHelper;
using TeeJee.ProcessHelper;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

class LogListBox : Gtk.Box {

	// 定义log表
	public Gtk.TreeView treeview;
	private Gtk.TreeViewColumn col_date;
	private Gtk.TreeViewColumn col_action;
	private Gtk.TreeViewColumn col_result;
	private Gtk.TreeViewColumn col_user;
	// private Gtk.TreeViewColumn col_desc;
	private int treeview_sort_column_index = 0;
	private bool treeview_sort_column_date = true;

	private Gtk.Window parent_window;

	public LogListBox(Gtk.Window _parent_window) {

		log_debug("LogListBox: LogListBox()");

		// base(Gtk.Orientation.VERTICAL, 6); // issue with vala
		GLib.Object(orientation: Gtk.Orientation.VERTICAL, spacing: 6);         // work-around
		parent_window = _parent_window;
		margin = 6;

		init_treeview();

		log_debug("LogListBox: LogListBox(): exit");
	}

	private void init_treeview() {

		// treeview
		treeview = new TreeView();
		treeview.get_selection().mode = SelectionMode.MULTIPLE;
		treeview.headers_clickable = true;
		treeview.has_tooltip = true;
		treeview.set_rules_hint(true);

		// sw_backups
		var sw_backups = new ScrolledWindow(null, null);
		sw_backups.set_shadow_type(ShadowType.ETCHED_IN);
		sw_backups.add(treeview);
		sw_backups.expand = true;
		add(sw_backups);

		// col_date
		col_date = new TreeViewColumn();
		col_date.title = _("Date");
		col_date.clickable = true;
		col_date.resizable = true;
		col_date.spacing = 1;
		col_date.min_width = 200;

		var cell_backup_icon = new CellRendererPixbuf();
		cell_backup_icon.surface = IconManager.lookup_surface("clock", 16, treeview.scale_factor);
		// cell_backup_icon.xpad = 1;
		cell_backup_icon.xpad = 4;
		cell_backup_icon.ypad = 6;
		col_date.pack_start(cell_backup_icon, false);

		var cell_date = new CellRendererText();
		col_date.pack_start(cell_date, false);
		col_date.set_cell_data_func(cell_date, cell_date_render);

		treeview.append_column(col_date);

		col_date.clicked.connect(() => {
			if (treeview_sort_column_index == 0) {
				treeview_sort_column_date = !treeview_sort_column_date;
			} else {
				treeview_sort_column_index = 0;
				treeview_sort_column_date = true;
			}
			refresh();
		});

		// col_action
		col_action = new TreeViewColumn();
		col_action.title = _("Action");
		col_action.resizable = true;
		col_action.clickable = true;
		col_action.min_width = 80;

		var cell_system = new CellRendererText();
		cell_system.ellipsize = Pango.EllipsizeMode.END;
		col_action.pack_start(cell_system, false);
		col_action.set_cell_data_func(cell_system, cell_active_render);
		treeview.append_column(col_action);

		col_action.clicked.connect(() => {
			if (treeview_sort_column_index == 1) {
				treeview_sort_column_date = !treeview_sort_column_date;
			} else {
				treeview_sort_column_index = 1;
				treeview_sort_column_date = false;
			}
			refresh();
		});

		// col_user
		var col = new TreeViewColumn();
		col.title = _("User");
		col.resizable = true;
		col.min_width = 80;
		col.clickable = true;
		var cell_size = new CellRendererText();
		cell_size.ellipsize = Pango.EllipsizeMode.END;
		// cell_size.xalign = (float) 1.0;
		col.pack_start(cell_size, false);
		col.set_cell_data_func(cell_size, cell_user_render);
		col_user = col;
		treeview.append_column(col_user);

		col_user.clicked.connect(() => {
			if (treeview_sort_column_index == 2) {
				treeview_sort_column_date = !treeview_sort_column_date;
			} else {
				treeview_sort_column_index = 2;
				treeview_sort_column_date = false;
			}
			refresh();
		});

		// col_result
		col_result = new TreeViewColumn();
		col_result.title = _("Result");
		col_result.resizable = true;
		// col_result.min_width = 80;
		col_result.clickable = true;
		var cell_tags = new CellRendererText();
		cell_tags.ellipsize = Pango.EllipsizeMode.END;
		col_result.pack_start(cell_tags, false);
		col_result.set_cell_data_func(cell_tags, cell_result_render);
		treeview.append_column(col_result);

		col_result.clicked.connect(() => {
			if (treeview_sort_column_index == 2) {
				treeview_sort_column_date = !treeview_sort_column_date;
			} else {
				treeview_sort_column_index = 2;
				treeview_sort_column_date = false;
			}
			refresh();
		});
	}

	// renderers

	private void cell_date_render(CellLayout cell_layout, CellRenderer cell, TreeModel model, TreeIter iter) {

		OperateLog bak;
		model.get(iter, 0, out bak, -1);

		var ctxt = (cell as Gtk.CellRendererText);
		ctxt.text = bak.date_formatted;
		// ctxt.sensitive = !bak.marked_for_deletion;

		ctxt.markup = "<b>%s</b>".printf(ctxt.text);

		// Note: Avoid AM/PM as it may be hidden due to locale settings
	}

	private void cell_result_render(CellLayout cell_layout, CellRenderer cell, TreeModel model, TreeIter iter) {

		OperateLog bak;
		model.get(iter, 0, out bak, -1);

		var ctxt = (cell as Gtk.CellRendererText);
		ctxt.text = bak.result;
		// ctxt.sensitive = !bak.marked_for_deletion;

		ctxt.markup = "<b>%s</b>".printf(ctxt.text);
	}

	private void cell_user_render(CellLayout cell_layout, CellRenderer cell, TreeModel model, TreeIter iter) {

		OperateLog bak;
		model.get(iter, 0, out bak, -1);

		var ctxt = (cell as Gtk.CellRendererText);
		ctxt.text = bak.user;
		// ctxt.sensitive = !bak.marked_for_deletion;

		ctxt.markup = "<b>%s</b>".printf(ctxt.text);
	}

	private void cell_active_render(CellLayout cell_layout, CellRenderer cell, TreeModel model, TreeIter iter) {
		OperateLog bak;
		model.get(iter, 0, out bak, -1);

		var ctxt = (cell as Gtk.CellRendererText);
		ctxt.text = bak.action;

		// ctxt.sensitive = !bak.marked_for_deletion;

		ctxt.markup = "<b>%s</b>".printf(ctxt.text);
	}

	// actions

	public void refresh() {
		log_debug("LogListBOx: refresh()");
		var model = new Gtk.ListStore(1, typeof (OperateLog));

		if (App.logRepo == null) {
			treeview.set_model(model);
			return;
		}

		App.logRepo.load_operateLog();

		var list = App.logRepo.operateLogs;
		log_debug("LogListBOx: list size %d".printf(list.size));
		if (treeview_sort_column_index == 0) {

			if (treeview_sort_column_date) {
				list.sort((a, b) => {
					OperateLog t1 = (OperateLog) a;
					OperateLog t2 = (OperateLog) b;

					return (t1.date.compare(t2.date));
				});
			} else {
				list.sort((a, b) => {
					OperateLog t1 = (OperateLog) a;
					OperateLog t2 = (OperateLog) b;

					return -1 * (t1.date.compare(t2.date));
				});
			}
		}


		TreeIter iter;
		// 将快照信息按行加入表中
		foreach (OperateLog bak in list) {
			model.append(out iter);
			model.set(iter, 0, bak);
		}

		treeview.set_model(model);
		treeview.columns_autosize();
		log_debug("LogListBOx: refresh() exit");
	}
}