/*
 * LogWindow.vala
 *
 * Copyright 2012-2018 Tony George <teejeetech@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */

using Gtk;
using Gee;

using TeeJee.Logging;
using TeeJee.FileSystem;
using TeeJee.JsonHelper;
using TeeJee.ProcessHelper;
using TeeJee.GtkHelper;
using TeeJee.System;
using TeeJee.Misc;

class LogWindow : Gtk.Window {

	private Gtk.Box vbox_main;
	private Gtk.Notebook notebook;
	private Gtk.ButtonBox bbox_action;

	// tabs
	private LogListBox log_list_box;

	private uint tmr_init;
	private int def_width = 500;
	private int def_height = 400;
	private bool success = false;

	public LogWindow() {

		log_debug("LogWindow: LogWindow()");

		this.title = _("List Operate Log");
		this.window_position = WindowPosition.CENTER;
		this.modal = true;
		this.set_default_size(def_width, def_height);
		this.icon = IconManager.lookup("timeshift", 16);

		this.delete_event.connect(on_delete_event);

		this.resize(def_width, def_height);

		// vbox_main
		vbox_main = new Gtk.Box(Orientation.VERTICAL, 6);
		vbox_main.margin = 0;
		add(vbox_main);

		// add notebook
		notebook = add_notebook(vbox_main, false, false);

		// create tab
		var vbox_tab = new Gtk.Box(Orientation.VERTICAL, 6);
		vbox_tab.margin = 12;

		// add_label_header(vbox_tab, _("Select Operate Log"), true);
		var label = new Gtk.Label(_("Operate Log"));

		log_list_box = new LogListBox(this);
		// log_list_box.hide_context_menu();
		log_list_box.margin = 0;
		vbox_tab.add(log_list_box);

		notebook.append_page(vbox_tab, label);

		create_actions();

		show_all();

		// go_first();

		tmr_init = Timeout.add(100, init_delayed);

		log_debug("LogWindow: LogWindow(): exit");
	}

	private bool init_delayed() {
		if (tmr_init > 0) {
			Source.remove(tmr_init);
			tmr_init = 0;
		}
		initialize_tab();

		return false;
	}

	private bool on_delete_event(Gdk.EventAny event) {
		return false;         // close window
	}

	private void create_actions() {
		var hbox = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 6);
		vbox_main.add(hbox);

		var bbox = new Gtk.ButtonBox(Gtk.Orientation.HORIZONTAL);
		bbox.margin = 12;
		bbox.spacing = 6;
		bbox.hexpand = true;
		hbox.add(bbox);

		bbox_action = bbox;

		bbox.set_layout(Gtk.ButtonBoxStyle.CENTER);

		Gtk.SizeGroup size_group = null;
	}

	private void initialize_tab() {
		log_debug("OperateLogWindow: initialize_tab()");
		if (notebook.page < 0) {
			return;
		}

		log_msg("");
		log_debug("page: %d".printf(notebook.page));

		// actions
		log_list_box.refresh();
		log_debug("OperateLogWindow: initialize_tab() exit");
	}
}